/*
    C++ Nelder-Meade Simplex Algorithm for Optimization
    Copyright (C) 2008  Colin Caprani - www.colincaprani.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// this is the main.cpp file for the NelderMead program

#include <iostream>
#include <vector>
#include <math.h>
#include "Simplex.h"

using namespace std;

double f1(vector<double> x);
double f2(vector<double> x);
double f3(vector<double> x);

void main()
{
	const int MAX_ITER = 5000;
	const double TOL = 1e-8;

//	vector<double> start(2,0.0);
//	start[0] = -1.2; start[1] = 1.0;
//	CSimplex NMS(2,TOL,MAX_ITER,&f1);

//	vector<double> start(4,0.0);
//	start[0] = 3; start[1] = -1.0; start[2] = 0.0; start[3] = 1.0;
//	CSimplex NMS(4,TOL,MAX_ITER,&f2);

	vector<double> start(3,0.0);
	start[0] = -1.0; start[1] = 2.0; start[2] = 1.0;
	CSimplex NMS(3,TOL,MAX_ITER,&f3);

	double result = NMS.amoeba(start);

}

// Rosenbrocks parabolic valley
double f1(vector<double> x)
{
	double y = 100*pow( (x[1] - x[0]*x[0]) ,2) + pow( (1-x[0]) ,2);

	return y;
}

// Powell's quartic function
double f2(vector<double> x)
{
	double y = pow( (x[0]+10*x[1]), 2) + 5*pow( (x[2]-x[3]) ,2) + pow( (x[1]-2*x[2]) ,4) + 10*pow( (x[0]-x[3]) ,4);

	return y;
}

// Fletcher and Powell's helical valley
double f3(vector<double> x)
{
	double PI = 4*atan(1);
	double theta_term = x[0] > 0 ? (5/PI)*atan(x[1]/x[0]) : PI + (5/PI)*atan(x[1]/x[0]);
	double y = 100*pow( (x[2]-theta_term) ,2) + pow( sqrt(x[0]*x[0]+x[1]*x[1]) ,2) + x[2]*x[2];

	return y;
}